# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from uuid import UUID
from bxb.models.aggregation_type import AggregationType
from typing import Optional, Set
from typing_extensions import Self

class BillableMetricResponse(BaseModel):
    """
    BillableMetricResponse
    """ # noqa: E501
    id: UUID
    code: StrictStr
    name: StrictStr
    description: Optional[StrictStr]
    aggregation_type: AggregationType
    field_name: Optional[StrictStr]
    recurring: StrictBool
    rounding_function: Optional[StrictStr]
    rounding_precision: Optional[StrictInt]
    expression: Optional[StrictStr]
    created_at: datetime
    updated_at: datetime
    __properties: ClassVar[List[str]] = ["id", "code", "name", "description", "aggregation_type", "field_name", "recurring", "rounding_function", "rounding_precision", "expression", "created_at", "updated_at"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BillableMetricResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if field_name (nullable) is None
        # and model_fields_set contains the field
        if self.field_name is None and "field_name" in self.model_fields_set:
            _dict['field_name'] = None

        # set to None if rounding_function (nullable) is None
        # and model_fields_set contains the field
        if self.rounding_function is None and "rounding_function" in self.model_fields_set:
            _dict['rounding_function'] = None

        # set to None if rounding_precision (nullable) is None
        # and model_fields_set contains the field
        if self.rounding_precision is None and "rounding_precision" in self.model_fields_set:
            _dict['rounding_precision'] = None

        # set to None if expression (nullable) is None
        # and model_fields_set contains the field
        if self.expression is None and "expression" in self.model_fields_set:
            _dict['expression'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BillableMetricResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "code": obj.get("code"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "aggregation_type": obj.get("aggregation_type"),
            "field_name": obj.get("field_name"),
            "recurring": obj.get("recurring"),
            "rounding_function": obj.get("rounding_function"),
            "rounding_precision": obj.get("rounding_precision"),
            "expression": obj.get("expression"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at")
        })
        return _obj


