# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict
from typing import Any, ClassVar, Dict, List
from bxb.models.portal_next_billing_info import PortalNextBillingInfo
from bxb.models.portal_quick_actions import PortalQuickActions
from bxb.models.portal_upcoming_charge import PortalUpcomingCharge
from bxb.models.portal_usage_progress import PortalUsageProgress
from typing import Optional, Set
from typing_extensions import Self

class PortalDashboardSummaryResponse(BaseModel):
    """
    Aggregated dashboard summary for the portal.
    """ # noqa: E501
    next_billing: List[PortalNextBillingInfo]
    upcoming_charges: List[PortalUpcomingCharge]
    usage_progress: List[PortalUsageProgress]
    quick_actions: PortalQuickActions
    __properties: ClassVar[List[str]] = ["next_billing", "upcoming_charges", "usage_progress", "quick_actions"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PortalDashboardSummaryResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in next_billing (list)
        _items = []
        if self.next_billing:
            for _item_next_billing in self.next_billing:
                if _item_next_billing:
                    _items.append(_item_next_billing.to_dict())
            _dict['next_billing'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in upcoming_charges (list)
        _items = []
        if self.upcoming_charges:
            for _item_upcoming_charges in self.upcoming_charges:
                if _item_upcoming_charges:
                    _items.append(_item_upcoming_charges.to_dict())
            _dict['upcoming_charges'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in usage_progress (list)
        _items = []
        if self.usage_progress:
            for _item_usage_progress in self.usage_progress:
                if _item_usage_progress:
                    _items.append(_item_usage_progress.to_dict())
            _dict['usage_progress'] = _items
        # override the default output from pydantic by calling `to_dict()` of quick_actions
        if self.quick_actions:
            _dict['quick_actions'] = self.quick_actions.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PortalDashboardSummaryResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "next_billing": [PortalNextBillingInfo.from_dict(_item) for _item in obj["next_billing"]] if obj.get("next_billing") is not None else None,
            "upcoming_charges": [PortalUpcomingCharge.from_dict(_item) for _item in obj["upcoming_charges"]] if obj.get("upcoming_charges") is not None else None,
            "usage_progress": [PortalUsageProgress.from_dict(_item) for _item in obj["usage_progress"]] if obj.get("usage_progress") is not None else None,
            "quick_actions": PortalQuickActions.from_dict(obj["quick_actions"]) if obj.get("quick_actions") is not None else None
        })
        return _obj


