# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class CustomerHealthStatus(str, Enum):
    """
    CustomerHealthStatus
    """

    """
    allowed enum values
    """
    GOOD = 'good'
    WARNING = 'warning'
    CRITICAL = 'critical'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CustomerHealthStatus from a JSON string"""
        return cls(json.loads(json_str))


