# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from uuid import UUID
from bxb.models.plan_summary import PlanSummary
from bxb.models.subscription_status import SubscriptionStatus
from typing import Optional, Set
from typing_extensions import Self

class PortalSubscriptionResponse(BaseModel):
    """
    Subscription response enriched with plan details for portal display.
    """ # noqa: E501
    id: UUID
    external_id: StrictStr
    status: SubscriptionStatus
    started_at: Optional[datetime]
    canceled_at: Optional[datetime]
    paused_at: Optional[datetime]
    downgraded_at: Optional[datetime]
    created_at: datetime
    plan: PlanSummary
    pending_downgrade_plan: Optional[PlanSummary] = None
    __properties: ClassVar[List[str]] = ["id", "external_id", "status", "started_at", "canceled_at", "paused_at", "downgraded_at", "created_at", "plan", "pending_downgrade_plan"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PortalSubscriptionResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of plan
        if self.plan:
            _dict['plan'] = self.plan.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pending_downgrade_plan
        if self.pending_downgrade_plan:
            _dict['pending_downgrade_plan'] = self.pending_downgrade_plan.to_dict()
        # set to None if started_at (nullable) is None
        # and model_fields_set contains the field
        if self.started_at is None and "started_at" in self.model_fields_set:
            _dict['started_at'] = None

        # set to None if canceled_at (nullable) is None
        # and model_fields_set contains the field
        if self.canceled_at is None and "canceled_at" in self.model_fields_set:
            _dict['canceled_at'] = None

        # set to None if paused_at (nullable) is None
        # and model_fields_set contains the field
        if self.paused_at is None and "paused_at" in self.model_fields_set:
            _dict['paused_at'] = None

        # set to None if downgraded_at (nullable) is None
        # and model_fields_set contains the field
        if self.downgraded_at is None and "downgraded_at" in self.model_fields_set:
            _dict['downgraded_at'] = None

        # set to None if pending_downgrade_plan (nullable) is None
        # and model_fields_set contains the field
        if self.pending_downgrade_plan is None and "pending_downgrade_plan" in self.model_fields_set:
            _dict['pending_downgrade_plan'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PortalSubscriptionResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "external_id": obj.get("external_id"),
            "status": obj.get("status"),
            "started_at": obj.get("started_at"),
            "canceled_at": obj.get("canceled_at"),
            "paused_at": obj.get("paused_at"),
            "downgraded_at": obj.get("downgraded_at"),
            "created_at": obj.get("created_at"),
            "plan": PlanSummary.from_dict(obj["plan"]) if obj.get("plan") is not None else None,
            "pending_downgrade_plan": PlanSummary.from_dict(obj["pending_downgrade_plan"]) if obj.get("pending_downgrade_plan") is not None else None
        })
        return _obj


