# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from uuid import UUID
from bxb.models.amount import Amount
from bxb.models.quantity import Quantity
from bxb.models.unit_price import UnitPrice
from typing import Optional, Set
from typing_extensions import Self

class InvoiceLineItem(BaseModel):
    """
    InvoiceLineItem
    """ # noqa: E501
    description: StrictStr
    quantity: Quantity
    unit_price: UnitPrice
    amount: Amount
    charge_id: Optional[UUID] = None
    metric_code: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["description", "quantity", "unit_price", "amount", "charge_id", "metric_code"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of InvoiceLineItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of quantity
        if self.quantity:
            _dict['quantity'] = self.quantity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of unit_price
        if self.unit_price:
            _dict['unit_price'] = self.unit_price.to_dict()
        # override the default output from pydantic by calling `to_dict()` of amount
        if self.amount:
            _dict['amount'] = self.amount.to_dict()
        # set to None if charge_id (nullable) is None
        # and model_fields_set contains the field
        if self.charge_id is None and "charge_id" in self.model_fields_set:
            _dict['charge_id'] = None

        # set to None if metric_code (nullable) is None
        # and model_fields_set contains the field
        if self.metric_code is None and "metric_code" in self.model_fields_set:
            _dict['metric_code'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of InvoiceLineItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "description": obj.get("description"),
            "quantity": Quantity.from_dict(obj["quantity"]) if obj.get("quantity") is not None else None,
            "unit_price": UnitPrice.from_dict(obj["unit_price"]) if obj.get("unit_price") is not None else None,
            "amount": Amount.from_dict(obj["amount"]) if obj.get("amount") is not None else None,
            "charge_id": obj.get("charge_id"),
            "metric_code": obj.get("metric_code")
        })
        return _obj


