# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from bxb.models.revenue_data_point import RevenueDataPoint
from bxb.models.trend_indicator import TrendIndicator
from typing import Optional, Set
from typing_extensions import Self

class RevenueResponse(BaseModel):
    """
    RevenueResponse
    """ # noqa: E501
    mrr_cents: Union[StrictFloat, StrictInt]
    total_revenue_this_month_cents: Union[StrictFloat, StrictInt]
    outstanding_invoices_cents: Union[StrictFloat, StrictInt]
    overdue_amount_cents: Union[StrictFloat, StrictInt]
    currency: StrictStr
    monthly_trend: List[RevenueDataPoint]
    mrr_trend: Optional[TrendIndicator] = None
    __properties: ClassVar[List[str]] = ["mrr_cents", "total_revenue_this_month_cents", "outstanding_invoices_cents", "overdue_amount_cents", "currency", "monthly_trend", "mrr_trend"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RevenueResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in monthly_trend (list)
        _items = []
        if self.monthly_trend:
            for _item_monthly_trend in self.monthly_trend:
                if _item_monthly_trend:
                    _items.append(_item_monthly_trend.to_dict())
            _dict['monthly_trend'] = _items
        # override the default output from pydantic by calling `to_dict()` of mrr_trend
        if self.mrr_trend:
            _dict['mrr_trend'] = self.mrr_trend.to_dict()
        # set to None if mrr_trend (nullable) is None
        # and model_fields_set contains the field
        if self.mrr_trend is None and "mrr_trend" in self.model_fields_set:
            _dict['mrr_trend'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RevenueResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "mrr_cents": obj.get("mrr_cents"),
            "total_revenue_this_month_cents": obj.get("total_revenue_this_month_cents"),
            "outstanding_invoices_cents": obj.get("outstanding_invoices_cents"),
            "overdue_amount_cents": obj.get("overdue_amount_cents"),
            "currency": obj.get("currency"),
            "monthly_trend": [RevenueDataPoint.from_dict(_item) for _item in obj["monthly_trend"]] if obj.get("monthly_trend") is not None else None,
            "mrr_trend": TrendIndicator.from_dict(obj["mrr_trend"]) if obj.get("mrr_trend") is not None else None
        })
        return _obj


