# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from uuid import UUID
from bxb.models.api_key_create_response import ApiKeyCreateResponse
from typing import Optional, Set
from typing_extensions import Self

class OrganizationCreateResponse(BaseModel):
    """
    Organization creation response that includes the initial API key.
    """ # noqa: E501
    id: UUID
    name: StrictStr
    default_currency: StrictStr
    timezone: StrictStr
    hmac_key: Optional[StrictStr]
    logo_url: Optional[StrictStr]
    portal_accent_color: Optional[StrictStr]
    portal_welcome_message: Optional[StrictStr]
    created_at: datetime
    updated_at: datetime
    api_key: ApiKeyCreateResponse
    __properties: ClassVar[List[str]] = ["id", "name", "default_currency", "timezone", "hmac_key", "logo_url", "portal_accent_color", "portal_welcome_message", "created_at", "updated_at", "api_key"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OrganizationCreateResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of api_key
        if self.api_key:
            _dict['api_key'] = self.api_key.to_dict()
        # set to None if hmac_key (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_key is None and "hmac_key" in self.model_fields_set:
            _dict['hmac_key'] = None

        # set to None if logo_url (nullable) is None
        # and model_fields_set contains the field
        if self.logo_url is None and "logo_url" in self.model_fields_set:
            _dict['logo_url'] = None

        # set to None if portal_accent_color (nullable) is None
        # and model_fields_set contains the field
        if self.portal_accent_color is None and "portal_accent_color" in self.model_fields_set:
            _dict['portal_accent_color'] = None

        # set to None if portal_welcome_message (nullable) is None
        # and model_fields_set contains the field
        if self.portal_welcome_message is None and "portal_welcome_message" in self.model_fields_set:
            _dict['portal_welcome_message'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OrganizationCreateResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "default_currency": obj.get("default_currency"),
            "timezone": obj.get("timezone"),
            "hmac_key": obj.get("hmac_key"),
            "logo_url": obj.get("logo_url"),
            "portal_accent_color": obj.get("portal_accent_color"),
            "portal_welcome_message": obj.get("portal_welcome_message"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at"),
            "api_key": ApiKeyCreateResponse.from_dict(obj["api_key"]) if obj.get("api_key") is not None else None
        })
        return _obj


